/* line.c
 * Copyright (C) 2011, 2012 Zykov Anton
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free 
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA 
 */

/*
   Создание объекта - линия.

   1) дескриптор (dsc),
   2) первая точка (p1),
   3) вторая точка (p2),
   4) длина линии (length),
   5) цвет линии (pixval),
   6) предыдущий узел (* prev),
   7) следующий узел (* next).
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <X11/Xlib.h>

#include "h_w_common.h"
#include "h_w_tool.h"
#include "h_line.h"
#include "h_o_cursor.h"



static pLINE line_lookup (pLINE * l, float x1, float y1, float x2, float y2);


/* Создание линии.
   Аргумент "l" - указатель на начало списка линий.
   Функция возвращает указатель на вставленный
   элемент "LINE". */
pLINE line_draw (pLINE * l, unsigned long pixval, float x1, float y1, float x2, float y2)
{
  static unsigned long dsc = 0; /* Дескриптор. */
  register pLINE new; /* Новый элемент списка. */
  /* XColor c; */
  
  if ((new = malloc (sizeof (struct LINE))) == NULL) return (0);
  new->dsc = dsc++;
  new->pixval = pixval;
  win_common_get_RGB_val (pixval, &new->r, &new->g, &new->b);
  new->r *= step;
  new->g *= step;
  new->b *= step;  
  //  fprintf (stdout, "%s %f %f %f %f \n", "LINE", x1, y1, x2, y2);
  new->p1 = point_add (&lPoints, x1, y1);
  new->p2 = point_add (&lPoints, x2, y2);
  new->length = sqrtf ( powf ((x2 - x1), 2.0) + powf ((y2 - y1), 2.0) );


  /* Если "l" == NULL (список пуст). */
  if (*l == NULL) {
    new->next = NULL;
    *l = new;
    return (new);
  }

  new->next = *l;
  (*l) = new;

  return (new);
}

/* Удаление линии с соответствующими координатами
   из списка.
   Функция возвращает "0" если линия найдена в 
   списке и удалена, "1" - если не найдена. */
int line_delete4 (pLINE * line, float x1, float y1, float x2, float y2)
{
  if (*line == NULL) return (0);
  register pLINE prev = NULL;
  register pLINE l;
  
  for (l = *line; l != NULL; l = l->next) {
    if ((l->p1->pt.x == x1) && (l->p1->pt.y == y1) &&
	(l->p2->pt.x == x2) && (l->p2->pt.y == y2)) {
      if (prev) {
	prev->next = l->next;
      } else {
	(*line) = l->next;
      }

      free (l->p1);
      free (l->p2);      
      free (l);

      return (0);
    }

    prev = l;
  }

  return (1);
}

/* Удаление линии с соответствующим дескриптором
   из списка.
   Функция возвращает "0" если линия найдена в 
   списке и удалена, "1" - если не найдена. */
int line_delete (pLINE * line, unsigned long dsc)
{
  if (*line == NULL) return (0);
  register pLINE prev = NULL;
  register pLINE l;
  
  for (l = *line; l != NULL; l = l->next) {
    if (l->dsc == dsc) {
      if (prev) {
	prev->next = l->next;
      } else {
	(*line) = l->next;
      }

      free (l->p1);
      free (l->p2);      
      free (l);

      return (0);
    }

    prev = l;
  }
  
  return (1);
}

/* Удаление списка линий.
   Функция возвращает "0". */
int line_delete_all (pLINE * line)
{
  if (*line == NULL) return (0);
  register pLINE l;
  
  for (l = *line; l != NULL; l = l->next) {
    free (l->p1);
    free (l->p2);      
    free (l);
  }

  (*line) = NULL;
  return (0);
}

/* Поиск элемента в списке с координатами
   "x1", "y1", "x2", "y2" .
   Функция возвращает указатель на найденный
   элемент, или "0". */
static pLINE line_lookup (pLINE * line, float x1, float y1, float x2, float y2)
{
  if (*line == NULL) return (0);
  register pLINE l;

  for (l = *line ; l != NULL; l = l->next) {
    if ((l->p1->pt.x == x1) && (l->p1->pt.y == y1) &&
	(l->p2->pt.x == x2) && (l->p2->pt.y == y2)) {
      return (l);
    }
  }

  return (0);
}

/* */
void line_test_print_certain (pLINE * line, float x1, float y1, float x2, float y2)
{
  pLINE l;
  l = line_lookup (line, x1, y1, x2, y2);
  if (!l) return;
    
  printf ("dsc:%ld  x1:%f  y1:%f  x2:%f  y2:%f length:%f \n",
	  l->dsc, l->p1->pt.x, l->p1->pt.y, l->p2->pt.x, l->p2->pt.y, l->length);
}

/* Просмотр всех элементов списка.
   pLINE - указатель на начало списка. */
void line_test_print_all (pLINE * line)
{
  if (*line == NULL) return;
  register pLINE l;

  for (l = *line; l != NULL; l = l->next)
    printf ("dsc:%ld  x1:%f  y1:%f  x2:%f  y2:%f length:%f \n",
	    l->dsc, l->p1->pt.x, l->p1->pt.y, l->p2->pt.x, l->p2->pt.y, l->length);
  fflush (stdout);
}


